/*
 * File:   main.c
 *
 * Boombox Transmission Module
 *
 * Cheng Xing, Rene Zhang
 * cx34, rz99
 * December 2015
 */

// === Config ==================================================================
#include "config.h"

// === Libraries ===============================================================
// Standard libraries
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

// Ring Buffer Library
#include "ring_buf.h"

// Audio Hardware Library
#include "adc.h"

// Transmission Library
#include "audio_transmit.h"

// Compression Library
#include "adpcm.h"

// === Audio Ring Buffers ======================================================
#define RING_SIZE 4096
// Circular ring buffer for audio
volatile static ring_buf_t ringL;
volatile static ring_buf_t ringR;

// === Transmission ============================================================
void transmit() {
  int i;
  char pkt[PKT_SIZE];
  unsigned char data;

  for (i = 0; i < PKT_SIZE; i++) {
    // Combine left and right stereo data into one byte
    data = (ring_pop(ringL) << 4) | ring_pop(ringR);
    pkt[i] = data;
  }
  at_transmit(pkt);
}

// === Timer 3 Interrupt Handler ===============================================
/*
 * Interrupt handler for timer 3.
 * ADC read is triggerd on timer interrupt
 */

// Which side are we currently reading
volatile int ADC_LR = 0;
// Currently read data
volatile unsigned short data;

// Pointers for encoding on left side
volatile unsigned long ps_encL = 0;
volatile int pi_encL = 0;
// Pointers for encoding on right side
volatile unsigned long ps_encR = 0;
volatile int pi_encR = 0;

void __ISR(_TIMER_3_VECTOR, ipl2) Timer3Handler(void) {
  if (ADC_LR) {
    adc_read(ADC_LEFT, data);
    ring_push(ringL, ADPCMEncoder(data - 32768, &ps_encL, &pi_encL));
  } else {
    adc_read(ADC_RIGHT, data);
    ring_push(ringR, ADPCMEncoder(data - 32768, &ps_encR, &pi_encR));
  }
  ADC_LR = 1 - ADC_LR;

  // Clear interrupt
  mT3ClearIntFlag();
}

// === Main  ===================================================================

int main(void) {
  // === Setup chip ============================================================
  SYSTEMConfig(sys_clock, SYS_CFG_WAIT_STATES | SYS_CFG_PCACHE);

  ANSELA =0; //make sure analog is cleared
  ANSELB =0;

  // === Setup input ring buffer ===============================================
  ringL = ring_new(RING_SIZE);
  ringR = ring_new(RING_SIZE);

  // === Setup timer3 ==========================================================
  OpenTimer3(T3_ON | T3_SOURCE_INT | T3_PS_1_1, sys_clock / ADC_FREQ / 2);

  ConfigIntTimer3(T3_INT_ON | T3_INT_PRIOR_2);
  mT3ClearIntFlag();

  // === Setup ADC =============================================================
  adc_setup();

  // === Setup Transmission ====================================================
  at_setup();

  // === setup system wide interrupts  =========================================
  INTEnableSystemMultiVectoredInt();

  // === Blinking debug LED ====================================================
  mPORTASetPinsDigitalOut(BIT_0);
  mPORTASetBits(BIT_0);
  int blink = 0;

  // === Transmission loop =====================================================
  while(1) {
    while (ring_size(ringL) < PKT_SIZE && ring_size(ringR) < PKT_SIZE);
    transmit();

    // Toggle debug LED
    if (blink++ % 4 == 0) {
      mPORTAToggleBits(BIT_0);
    }
  }
} // main
